# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.bindings.stix_common as common_binding


class KillChain(stix.Entity):
    _binding = common_binding
    _namespace = 'http://stix.mitre.org/common-1'
    _binding_class = _binding.KillChainType

    def __init__(self, id_=None, name=None, definer=None, reference=None):
        self.id_ = id_
        self.name = name
        self.definer = definer
        self.reference = reference
        self.number_of_phases = None  # can we just do len(self.kill_chain_phases)?
        self.kill_chain_phases = None

    @property
    def kill_chain_phases(self):
        return self._kill_chain_phases

    @kill_chain_phases.setter
    def kill_chain_phases(self, value):
        self._kill_chain_phases = _KillChainPhases(value)

    def add_kill_chain_phase(self, value):
        self.kill_chain_phases.append(value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(KillChain, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.name = self.name
        return_obj.definer = self.definer
        return_obj.reference = self.reference
        return_obj.number_of_phases = self.number_of_phases
        return_obj.Kill_Chain_Phase = self.kill_chain_phases.to_obj(ns_info=ns_info)

        return return_obj

    def __eq__(self, other):
        if self is other:
            return True

        if not isinstance(other, self.__class__):
            return False

        return other.to_dict() == self.to_dict()

    def __ne__(self, other):
        return not self.__eq__(other)

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.name = obj.name
        return_obj.definer = obj.definer
        return_obj.reference = obj.reference
        return_obj.number_of_phases = obj.number_of_phases
        return_obj.kill_chain_phases = _KillChainPhases.from_obj(obj.Kill_Chain_Phase)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        get = d.get
        return_obj.id_ = get('id')
        return_obj.name = get('name')
        return_obj.definer = get('definer')
        return_obj.reference = get('reference')
        return_obj.number_of_phases = get('number_of_phases')
        return_obj.kill_chain_phases = \
            _KillChainPhases.from_dict(get('kill_chain_phases'))

        return return_obj


class KillChains(stix.EntityList):
    _binding = common_binding
    _namespace = 'http://stix.mitre.org/common-1'
    _binding_class = _binding.KillChainsType
    _contained_type = KillChain
    _binding_var = "Kill_Chain"
    _inner_name = "kill_chains"


class KillChainPhase(stix.Entity):
    _binding = common_binding
    _namespace = 'http://stix.mitre.org/common-1'
    _binding_class = _binding.KillChainPhaseType

    def __init__(self, phase_id=None, name=None, ordinality=None):
        self.phase_id = phase_id
        self.name = name
        self.ordinality = ordinality

    @property
    def phase_id(self):
        return self._phase_id

    @phase_id.setter
    def phase_id(self, value):
        self._set_var(basestring, try_cast=False, phase_id=value)

    @property
    def ordinality(self):
        return self._ordinality

    @ordinality.setter
    def ordinality(self, value):
        if value is not None:
            self._ordinality = int(value)
        else:
            self._ordinality = None

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()

        super(KillChainPhase, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        return_obj.phase_id = self.phase_id
        return_obj.name = self.name
        return_obj.ordinality = self.ordinality

        return return_obj

    def __eq__(self, other):
        if other is self:
            return True

        if not isinstance(other, KillChainPhase):
            return False

        return other.to_dict() == self.to_dict()

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(tuple(sorted(self.to_dict().items())))

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.phase_id = obj.phase_id
        return_obj.name = obj.name
        return_obj.ordinality = obj.ordinality

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.phase_id = d.get('phase_id')
        return_obj.name = d.get('name')
        return_obj.ordinality = d.get('ordinality')

        return return_obj

    def to_dict(self):
        return super(KillChainPhase, self).to_dict()


class KillChainPhaseReference(KillChainPhase):
    _binding = common_binding
    _namespace = 'http://stix.mitre.org/common-1'
    _binding_class = _binding.KillChainPhaseReferenceType

    def __init__(self, phase_id=None, name=None, ordinality=None, kill_chain_id=None, kill_chain_name=None):
        super(KillChainPhaseReference, self).__init__(phase_id, name, ordinality)
        self.kill_chain_id = kill_chain_id
        self.kill_chain_name = kill_chain_name

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()

        super(KillChainPhaseReference, self).to_obj(return_obj=return_obj, ns_info=ns_info)
        return_obj.kill_chain_id = self.kill_chain_id
        return_obj.kill_chain_name = self.kill_chain_name
        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        super(KillChainPhaseReference, cls).from_obj(obj, return_obj=return_obj)

        return_obj.kill_chain_id = obj.kill_chain_id
        return_obj.kill_chain_name = obj.kill_chain_name
        return return_obj

    def to_dict(self):
        return super(KillChainPhaseReference, self).to_dict()

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        super(KillChainPhaseReference, cls).from_dict(d, return_obj=return_obj)
        return_obj.kill_chain_id = d.get('kill_chain_id')
        return_obj.kill_chain_name = d.get('kill_chain_name')
        return return_obj


class KillChainPhasesReference(stix.EntityList):
    _binding = common_binding
    _namespace = 'http://stix.mitre.org/common-1'
    _binding_class = _binding.KillChainPhasesReferenceType
    _contained_type = KillChainPhaseReference
    _binding_var = "Kill_Chain_Phase"
    _inner_name = "kill_chain_phases"

    def _fix_value(self, value):
        if not isinstance(value, KillChainPhase):
            return super(KillChainPhasesReference, self)._fix_value(value)

        if value.phase_id:
            return KillChainPhaseReference(phase_id=value.phase_id)

        raise ValueError("KillChainPhase must have a phase_id.")


# NOT AN ACTUAL STIX TYPE!
class _KillChainPhases(stix.TypedList):
    _contained_type = KillChainPhase



# Copyright (c) 2016, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from stix.bindings import *
import stix.bindings.data_marking as data_marking_binding

XML_NS = "http://www.us-cert.gov/STIXMarkingStructure#AISConsentMarking-2"

#
# Data representation classes.
#


class AISMarkingStructure(data_marking_binding.MarkingStructureType):
    subclass = None
    superclass = data_marking_binding.MarkingStructureType

    xmlns = XML_NS
    xmlns_prefix = "AIS"
    xml_type = "AISMarkingStructure"

    def __init__(self, idref=None, marking_model_ref=None, marking_model_name=None, id=None, Is_Proprietary=None, Not_Proprietary=None):
        super(AISMarkingStructure, self).__init__(idref=idref, marking_model_ref=marking_model_ref, marking_model_name=marking_model_name, id=id)
        self.Is_Proprietary = Is_Proprietary
        self.Not_Proprietary = Not_Proprietary

    def factory(*args_, **kwargs_):
        if AISMarkingStructure.subclass:
            return AISMarkingStructure.subclass(*args_, **kwargs_)
        else:
            return AISMarkingStructure(*args_, **kwargs_)
    factory = staticmethod(factory)

    def get_Is_Proprietary(self):
        return self.Is_Proprietary

    def set_Is_Proprietary(self, Is_Proprietary):
        self.Is_Proprietary = Is_Proprietary

    def get_Not_Proprietary(self):
        return self.Not_Proprietary

    def set_Not_Proprietary(self, Not_Proprietary):
        self.Not_Proprietary = Not_Proprietary

    def hasContent_(self):
        if (
            self.Is_Proprietary is not None or
            self.Not_Proprietary is not None or
            super(AISMarkingStructure, self).hasContent_()
        ):
            return True
        else:
            return False

    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='AISMarkingStructure', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='AISMarkingStructure')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))

    def exportAttributes(self, lwrite, level, already_processed, namespace_=XML_NS, name_='AISMarkingStructure'):
        super(AISMarkingStructure, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='AISMarkingStructure')
        if 'xsi:type' not in already_processed:
            already_processed.add('xsi:type')
            xsi_type = " xsi:type='%s:%s'" % (self.xmlns_prefix, self.xml_type)
            lwrite(xsi_type)

    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='AISMarkingStructure', fromsubclass_=False, pretty_print=True):
        super(AISMarkingStructure, self).exportChildren(lwrite, level, nsmap, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Is_Proprietary is not None:
            self.Is_Proprietary.export(lwrite, level, nsmap, namespace_, name_='Is_Proprietary', pretty_print=pretty_print)
        if self.Not_Proprietary is not None:
            self.Not_Proprietary.export(lwrite, level, nsmap, namespace_, name_='Not_Proprietary', pretty_print=pretty_print)

    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)

    def buildAttributes(self, node, attrs, already_processed):
        super(AISMarkingStructure, self).buildAttributes(node, attrs, already_processed)

    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Is_Proprietary':
            obj_ = IsProprietary.factory()
            obj_.build(child_)
            self.set_Is_Proprietary(obj_)
        if nodeName_ == 'Not_Proprietary':
            obj_ = NotProprietary.factory()
            obj_.build(child_)
            self.set_Not_Proprietary(obj_)
        super(AISMarkingStructure, self).buildChildren(child_, node, nodeName_, True)
# end class AISMarkingStructure


class IsProprietary(GeneratedsSuper):
    subclass = None
    superclass = None

    def __init__(self, CISA_Proprietary=None, AISConsent=None, TLPMarking=None):
        self.CISA_Proprietary = _cast(bool, CISA_Proprietary)
        self.AISConsent = AISConsent
        self.TLPMarking = TLPMarking

    def factory(*args_, **kwargs_):
        if IsProprietary.subclass:
            return IsProprietary.subclass(*args_, **kwargs_)
        else:
            return IsProprietary(*args_, **kwargs_)
    factory = staticmethod(factory)

    def get_AISConsent(self):
        return self.AISConsent

    def set_AISConsent(self, AISConsent):
        self.AISConsent = AISConsent

    def get_TLPMarking(self):
        return self.TLPMarking

    def set_TLPMarking(self, TLPMarking):
        self.TLPMarking = TLPMarking

    def get_CISA_Proprietary(self):
        return self.CISA_Proprietary

    def set_CISA_Proprietary(self, CISA_Proprietary):
        self.CISA_Proprietary = CISA_Proprietary

    def hasContent_(self):
        if (
            self.AISConsent is not None or
            self.TLPMarking is not None
        ):
            return True
        else:
            return False

    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='IsProprietary', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='IsProprietary')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))

    def exportAttributes(self, lwrite, level, already_processed, namespace_=XML_NS, name_='IsProprietary'):
        if self.CISA_Proprietary is not None and 'CISA_Proprietary' not in already_processed:
            already_processed.add('CISA_Proprietary')
            lwrite(' CISA_Proprietary="%s"' % self.gds_format_boolean(self.CISA_Proprietary, input_name='CISA_Proprietary'))

    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='IsProprietary', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.AISConsent is not None:
            self.AISConsent.export(lwrite, level, nsmap, namespace_, name_='AISConsent', pretty_print=pretty_print)
        if self.TLPMarking is not None:
            self.TLPMarking.export(lwrite, level, nsmap, namespace_, name_='TLPMarking', pretty_print=pretty_print)

    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)

    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('CISA_Proprietary', node)
        if value is not None and 'CISA_Proprietary' not in already_processed:
            already_processed.add('CISA_Proprietary')
            if value in ('true', '1'):
                self.CISA_Proprietary = True
            elif value in ('false', '0'):
                self.CISA_Proprietary = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')

    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'AISConsent':
            obj_ = AISConsentType.factory()
            obj_.build(child_)
            self.set_AISConsent(obj_)
        elif nodeName_ == 'TLPMarking':
            obj_ = TLPMarkingType.factory()
            obj_.build(child_)
            self.set_TLPMarking(obj_)
# end class IsProprietary


class NotProprietary(GeneratedsSuper):
    subclass = None
    superclass = None

    def __init__(self, CISA_Proprietary=None, AISConsent=None, TLPMarking=None):
        self.CISA_Proprietary = _cast(bool, CISA_Proprietary)
        self.AISConsent = AISConsent
        self.TLPMarking = TLPMarking

    def factory(*args_, **kwargs_):
        if NotProprietary.subclass:
            return NotProprietary.subclass(*args_, **kwargs_)
        else:
            return NotProprietary(*args_, **kwargs_)
    factory = staticmethod(factory)

    def get_AISConsent(self):
        return self.AISConsent

    def set_AISConsent(self, AISConsent):
        self.AISConsent = AISConsent

    def get_TLPMarking(self):
        return self.TLPMarking

    def set_TLPMarking(self, TLPMarking):
        self.TLPMarking = TLPMarking

    def get_CISA_Proprietary(self):
        return self.CISA_Proprietary

    def set_CISA_Proprietary(self, CISA_Proprietary):
        self.CISA_Proprietary = CISA_Proprietary

    def hasContent_(self):
        if (
            self.AISConsent is not None or
            self.TLPMarking is not None
        ):
            return True
        else:
            return False

    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='NotProprietary', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='NotProprietary')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))

    def exportAttributes(self, lwrite, level, already_processed, namespace_=XML_NS, name_='NotProprietary'):
        if self.CISA_Proprietary is not None and 'CISA_Proprietary' not in already_processed:
            already_processed.add('CISA_Proprietary')
            lwrite(' CISA_Proprietary="%s"' % self.gds_format_boolean(self.CISA_Proprietary, input_name='CISA_Proprietary'))

    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='NotProprietary', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.AISConsent is not None:
            self.AISConsent.export(lwrite, level, nsmap, namespace_, name_='AISConsent', pretty_print=pretty_print)
        if self.TLPMarking is not None:
            self.TLPMarking.export(lwrite, level, nsmap, namespace_, name_='TLPMarking', pretty_print=pretty_print)

    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)

    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('CISA_Proprietary', node)
        if value is not None and 'CISA_Proprietary' not in already_processed:
            already_processed.add('CISA_Proprietary')
            if value in ('true', '1'):
                self.CISA_Proprietary = True
            elif value in ('false', '0'):
                self.CISA_Proprietary = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')

    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'AISConsent':
            obj_ = AISConsentType.factory()
            obj_.build(child_)
            self.set_AISConsent(obj_)
        elif nodeName_ == 'TLPMarking':
            obj_ = TLPMarkingType.factory()
            obj_.build(child_)
            self.set_TLPMarking(obj_)
# end class NotProprietary


class AISConsentType(GeneratedsSuper):
    subclass = None
    superclass = None

    def __init__(self, consent=None):
        self.consent = _cast(None, consent)
        pass

    def factory(*args_, **kwargs_):
        if AISConsentType.subclass:
            return AISConsentType.subclass(*args_, **kwargs_)
        else:
            return AISConsentType(*args_, **kwargs_)
    factory = staticmethod(factory)

    def get_consent(self):
        return self.consent

    def set_consent(self, consent):
        self.consent = consent

    def hasContent_(self):
        if (

        ):
            return True
        else:
            return False

    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='AISConsentType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='AISConsentType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))

    def exportAttributes(self, lwrite, level, already_processed, namespace_=XML_NS, name_='AISConsentType'):
        if self.consent is not None and 'consent' not in already_processed:
            already_processed.add('consent')
            lwrite(' consent=%s' % (quote_attrib(self.consent), ))

    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='AISConsentType', fromsubclass_=False, pretty_print=True):
        pass

    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)

    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('consent', node)
        if value is not None and 'consent' not in already_processed:
            already_processed.add('consent')
            self.consent = value

    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class AISConsentType


class TLPMarkingType(GeneratedsSuper):
    subclass = None
    superclass = None

    def __init__(self, color=None):
        self.color = _cast(None, color)
        pass

    def factory(*args_, **kwargs_):
        if TLPMarkingType.subclass:
            return TLPMarkingType.subclass(*args_, **kwargs_)
        else:
            return TLPMarkingType(*args_, **kwargs_)
    factory = staticmethod(factory)

    def get_color(self):
        return self.color

    def set_color(self, color):
        self.color = color

    def hasContent_(self):
        if (

        ):
            return True
        else:
            return False

    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='TLPMarkingType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='TLPMarkingType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))

    def exportAttributes(self, lwrite, level, already_processed, namespace_=XML_NS, name_='TLPMarkingType'):
        if self.color is not None and 'color' not in already_processed:
            already_processed.add('color')
            lwrite(' color=%s' % (quote_attrib(self.color), ))

    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='TLPMarkingType', fromsubclass_=False, pretty_print=True):
        pass

    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)

    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('color', node)
        if value is not None and 'color' not in already_processed:
            already_processed.add('color')
            self.color = value

    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class TLPMarkingType


data_marking_binding.add_extension(AISMarkingStructure)

GDSClassesMapping = {}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""


def usage():
    print USAGE_TEXT
    sys.exit(1)


def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass


def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'AISMarkingStructure'
        rootClass = AISMarkingStructure
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    # sys.stdout.write('<?xml version="1.0" ?>\n')
    # rootObj.export(sys.stdout, 0, name_=rootTag,
    #     namespacedef_='',
    #     pretty_print=True)
    return rootObj


def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'AISMarkingStructure'
        rootClass = AISMarkingStructure
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement


def parseString(inString):
    from StringIO import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'AISMarkingStructure'
        rootClass = AISMarkingStructure
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    # sys.stdout.write('<?xml version="1.0" ?>\n')
    # rootObj.export(sys.stdout, 0, name_="AISHandling",
    #     namespacedef_='')
    return rootObj


def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "NotProprietary",
    "IsProprietary"
    "AISConsentType",
    "AISMarkingStructure"
    "TLPMarkingType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.exploit_target as exploit_target_binding
from stix.common import StructuredText


class Weakness(stix.Entity):
    """Implementation of STIX ``Weakness``.

    Args:
        cwe_id(optional): Common Weakness Enumeration value as a string
        description (optional): A string description.

    """
    _binding = exploit_target_binding
    _binding_class = _binding.WeaknessType
    _namespace = "http://stix.mitre.org/ExploitTarget-1"

    def __init__(self, description=None, cwe_id=None):
        self.description = description
        self.cwe_id = cwe_id

    @property
    def cwe_id(self):
        """
        Common Weakness Enumeration value as a string
        """
        return self._cwe_id

    @cwe_id.setter
    def cwe_id(self, value):
        self._cwe_id = value

    @property
    def description(self):
        """The ``description`` property for this :class:`Weakness`.

        Default Value: ``None``

        Note:
            If set to a value that is not an instance of
            :class:`stix.common.structured_text.StructuredText`, an attempt to
            will be made to convert the value into an instance of
            :class:`stix.common.structured_text.StructuredText`.

        Returns:
            An instance of
            :class:`stix.common.structured_text.StructuredText`

        """
        return self._description

    @description.setter
    def description(self, value):
        self._set_var(StructuredText, description=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Weakness, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        return_obj.CWE_ID = self.cwe_id

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.cwe_id = obj.CWE_ID

        return return_obj

    def to_dict(self):
        return super(Weakness, self).to_dict()

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.cwe_id = dict_repr.get('cwe_id')

        return return_obj


class _Weaknesses(stix.TypedList):
    _contained_type = Weakness



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest
from stix.test import EntityTestCase
import stix.extensions.identity.ciq_identity_3_0 as ciq


class CIQIdentity3_0InstanceTests(EntityTestCase, unittest.TestCase):
    klass = ciq.CIQIdentity3_0Instance
    _full_dict = {
        'id': 'example:ciqidentity-1',
        'name': 'John Smith',
        'roles': ['Programmer', 'Analyst'],
        'specification': {
            'party_name': {
                'name_lines': [
                    {'value': 'Foo'},
                    {'value': 'Bar'}
                ],
                'organisation_names': [
                    {
                        'name_elements': [
                            {
                                'element_type': 'FullName',
                                'value': 'Foo Inc.'
                            }
                        ],
                        'subdivision_names': [
                            {
                                'type': 'Department',
                                'value': 'InfoSec'
                            }
                        ]
                    }
                ],
                'person_names': [
                    {
                        'name_elements': [
                            {'value': 'John Smith'}
                        ]
                    },
                    {
                        'name_elements': [
                            {'value': 'Jill Smith'}
                        ]
                    }
                ]
            },
            'languages': [
                {'value': 'test language'}
            ],
            'addresses': [
                {
                    'free_text_address': {
                        'address_lines': ['1234 Example Lane.']
                    },
                    'country': {
                        'name_elements': [
                            {
                                'value': 'name 1',
                                'name_code': 'US',
                                'name_code_type': 'ISO 3166-1 alpha-2'
                            },
                            {
                                'value': 'name 2',
                                'name_code': 'BZ',
                                'name_code_type': 'ISO 3166-1 alpha-2',
                                'name_type': 'ISO'
                            }
                        ]
                    },
                    'administrative_area': {
                        'name_elements': [
                            {'value': 'admin area name 1'},
                            {'value': 'admin area name 2'}
                        ]
                    }
                }
            ],
            'electronic_address_identifiers': [
                {
                    'type': 'EMAIL',
                    'value': 'an eai v'
                }
            ],
            'free_text_lines': [
                {
                    'type': 'ftl type',
                    'value': 'ftl value'
                }
            ],
            'contact_numbers': [
                {
                    'communication_media_type': 'Fax',
                    'contact_number_elements': [
                        {
                            'value': 'a contact number',
                            'type': 'Pin'
                        }
                    ]
                }
            ],
            'nationalities': [
                {
                    'name_elements': [
                        {'value': 'name 1'},
                        {'value': 'name 2'}
                    ]
                }
            ],
            'organisation_info': {
                'industry_type': 'SECTOR 1 | SECTOR 2',
            }
        },
        'xsi:type': 'stix-ciqidentity:CIQIdentity3.0InstanceType'
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.utils as utils
import stix.bindings.stix_common as common_binding

# relative
from .names import Names


class CampaignRef(stix.Entity):
    _namespace = "http://stix.mitre.org/common-1"
    _binding = common_binding
    _binding_class = common_binding.CampaignReferenceType

    def __init__(self, idref=None, timestamp=None, **kwargs):
        super(CampaignRef, self).__init__(**kwargs)
        self.idref = idref
        self.timestamp = timestamp
        self.names = None

    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = utils.dates.parse_value(value)

    @property
    def names(self):
        return self._names

    @names.setter
    def names(self, value):
        self._names = Names(value)

    def add_name(self, name):
        self.names.append(name)

    def to_obj(self, return_obj=None, ns_info=None):

        if not return_obj:
            return_obj = self._binding_class()

        return_obj = super(CampaignRef, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        return_obj.idref = self.idref
        return_obj.timestamp = utils.serialize_value(self.timestamp)

        if self.names:
            return_obj.Names = self.names.to_obj()

        return return_obj

    def to_dict(self):
        return super(CampaignRef, self).to_dict()

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.idref = obj.idref
        return_obj.timestamp = obj.timestamp
        return_obj.names = Names.from_obj(obj.Names)

        return return_obj

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        get = dict_repr.get
        return_obj.idref = get("idref")
        return_obj.timestamp = get("timestamp")
        return_obj.names = Names.from_dict(get("names"))

        return return_obj
